@@ -1,4 +1,4 @@
-/* 
+/*
  * How to test v4l2loopback:
  * 1. launch this test program (even in background), it will initialize the
  *    loopback device and keep it open so it won't loose the settings.
@@ -48,6 +48,9 @@
 # define FRAME_FORMAT V4L2_PIX_FMT_YVU420
 #endif
 
+static int debug=0;
+
+
 int format_properties(const unsigned int format,
 		const unsigned int width,
 		const unsigned int height,
@@ -75,6 +78,16 @@ size_t lw, fw;
 }
 
 
+void print_format(struct v4l2_format*vid_format) {
+  printf("	vid_format->type                =%d\n",	vid_format->type );
+  printf("	vid_format->fmt.pix.width       =%d\n",	vid_format->fmt.pix.width );
+  printf("	vid_format->fmt.pix.height      =%d\n",	vid_format->fmt.pix.height );
+  printf("	vid_format->fmt.pix.pixelformat =%d\n",	vid_format->fmt.pix.pixelformat);
+  printf("	vid_format->fmt.pix.sizeimage   =%d\n",	vid_format->fmt.pix.sizeimage );
+  printf("	vid_format->fmt.pix.field       =%d\n",	vid_format->fmt.pix.field );
+  printf("	vid_format->fmt.pix.bytesperline=%d\n",	vid_format->fmt.pix.bytesperline );
+  printf("	vid_format->fmt.pix.colorspace  =%d\n",	vid_format->fmt.pix.colorspace );
+}
 
 int main(int argc, char**argv)
 {
@@ -87,40 +100,28 @@ int main(int argc, char**argv)
 	__u8*buffer;
 	__u8*check_buffer;
 
-        const char*video_device=VIDEO_DEVICE;
+  const char*video_device=VIDEO_DEVICE;
+	int fdwr = 0;
+	int ret_code = 0;
 
-	if(!format_properties(FRAME_FORMAT,
-		FRAME_WIDTH, FRAME_HEIGHT,
-		&linewidth,
-		&framesize)) {
-		printf("unable to guess correct settings for format '%d'\n", FRAME_FORMAT);
-	}
-
-
-	buffer=(__u8*)malloc(sizeof(__u8)*framesize);
-	check_buffer=(__u8*)malloc(sizeof(__u8)*framesize);
+	int i;
 
 	if(argc>1) {
 		video_device=argv[1];
 		printf("using output device: %s\n", video_device);
 	}
 
-	int i;
-	memset(buffer, 0, framesize);
-	memset(check_buffer, 0, framesize);
-	for (i = 0; i < framesize; ++i) {
-		//buffer[i] = i % 2;
-		check_buffer[i] = 0;
-	}
-
-	int fdwr = open(video_device, O_RDWR);
+	fdwr = open(video_device, O_RDWR);
 	assert(fdwr >= 0);
 
-	int ret_code = ioctl(fdwr, VIDIOC_QUERYCAP, &vid_caps);
+	ret_code = ioctl(fdwr, VIDIOC_QUERYCAP, &vid_caps);
 	assert(ret_code != -1);
 
 	memset(&vid_format, 0, sizeof(vid_format));
 
+	ret_code = ioctl(fdwr, VIDIOC_G_FMT, &vid_format);
+  if(debug)print_format(&vid_format);
+
 	vid_format.type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
 	vid_format.fmt.pix.width = FRAME_WIDTH;
 	vid_format.fmt.pix.height = FRAME_HEIGHT;
@@ -129,10 +130,34 @@ int main(int argc, char**argv)
 	vid_format.fmt.pix.field = V4L2_FIELD_NONE;
 	vid_format.fmt.pix.bytesperline = linewidth;
 	vid_format.fmt.pix.colorspace = V4L2_COLORSPACE_SRGB;
+
+  if(debug)print_format(&vid_format);
 	ret_code = ioctl(fdwr, VIDIOC_S_FMT, &vid_format);
+
 	assert(ret_code != -1);
 
-	printf("frame: format=%d\tsize=%d\n", FRAME_FORMAT, framesize);
+	if(debug)printf("frame: format=%d\tsize=%d\n", FRAME_FORMAT, framesize);
+  print_format(&vid_format);
+
+	if(!format_properties(vid_format.fmt.pix.pixelformat,
+                        vid_format.fmt.pix.width, vid_format.fmt.pix.height,
+                        &linewidth,
+                        &framesize)) {
+		printf("unable to guess correct settings for format '%d'\n", FRAME_FORMAT);
+	}
+	buffer=(__u8*)malloc(sizeof(__u8)*framesize);
+	check_buffer=(__u8*)malloc(sizeof(__u8)*framesize);
+
+	memset(buffer, 0, framesize);
+	memset(check_buffer, 0, framesize);
+	for (i = 0; i < framesize; ++i) {
+		//buffer[i] = i % 2;
+		check_buffer[i] = 0;
+	}
+
+
+
+
 
 	write(fdwr, buffer, framesize);
 
